#!/usr/bin/env python
# -*- encoding: euc-jp -*-

# export.py

import time
from repository import Repository
from utils import *

ARCHIVE_URL = 'http://traiss.tabesugi.net/archives/%s/'


##  export_rss
##
def export_rss((title, link, description), items, editurl, puburl):
  print '<?xml version="1.0" encoding="%s"?>' % RSS_ENCODING
  print '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns="http://purl.org/rss/1.0/">'
  print ' <channel rdf:about="%s">' % puburl
  print '  <title>%s</title>' % rssquote(title)
  print '  <link>%s</link>' % rssquote(link)
  if description:
    print '  <description>%s</description>' % rssquote(description)
  print '  <items><rdf:Seq>'
  for (itemid,e0,e1) in items:
    print '   <rdf:li resource="%s" />' % rssquote(e0.link)
  print '  </rdf:Seq></items>'
  print ' </channel>'
  for (itemid,e0,e1) in items:
    print ' <item rdf:about="%s">' % rssquote(e0.link)
    title = remsp(dump_raw(e1.title))
    if e1.flag != 'P':
      title = u'[̤] %s' % title
    print '  <title>%s</title>' % rssquote(title)
    print '  <link>%s</link>' % rssquote(e0.link)
    url = editurl+itemid
    if e1.flag != 'P':
      description = u'<a href="%s">[ˤϤ]</a>' % url
    else:
      description = remsp(dump_raw(e1.description)) + u' <small><a href="%s">[Խ...]</a></small>' % url
    if description:
      print '  <description>%s</description>' % rssquote(description)
    print ' </item>'
  print '</rdf:RDF>'
  return


##  export_html
##
def export_html((title, link, description), items, editurl, archiveurl):
  def enc(s):
    return s.encode(HTML_ENCODING)
  print '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"><html><head>'
  print '<meta http-equiv="Content-Type" content="text/html; charset=%s">' % HTML_ENCODING
  print '<meta name="robots" content="nofollow">'
  print '''<style><!--
h1 { letter-spacing:0.2em; text-align:left; padding:8pt; background:#88f; }
h1 a { color: white; }
h2 { text-align:left; padding-left:8pt; padding-bottom:4pt; border-left:6pt solid brown; border-bottom:2pt solid brown; }
body { line-height: 130%; }
--></style>'''
  if title:
    print '<title>%s</title>' % htmlquote(title)
  print '</head><body>'
  if link and title:
    print '<h1><a href="%s">%s</a></h1>' % (htmlquote(link), htmlquote(title))
  if description:
    print '<p> %s' % description
  for (itemid,e0,e1) in items:
    title = remsp(dump_raw(e1.title))
    if e1.flag != 'P':
      title = u'[̤] %s' % title
    print '<h2>%s</h2>' % htmlquote(title)
    print '<p><a href="%s"><strong>%s</strong></a>&nbsp;' % (htmlquote(e0.link), remsp(dump_raw(e0.title)))
    print enc(u'<small>: %s</small>') % repr_time(e0.modtime, time.time())
    if e1.flag != 'P':
      url = editurl+itemid
      print enc(u'<p><a href="%s">[ˤϤ]</a>') % url
    else:
      description = remsp(dump_raw(e1.description))
      if description:
        print u'<p>%s' % description.encode(HTML_ENCODING, 'xmlcharrefreplace')
        print enc(u'<small><a href="%s">[Խ...]</a></small>' % (editurl+itemid))
  print enc(u'<hr noshade><center><a href="%s">[ε]</a></center>' % archiveurl)
  print '<hr noshade><address>Generated by <a href="%s">Traiss-%s</a></address>' % (TRAISS_TOPURL, TRAISS_VERSION)
  print '</body></html>'
  return


# main
def main():
  import sys, getopt
  def usage():
    print "usage: export.py {-h archiveurl | -r puburl} [-n n] dirname"
    sys.exit(2)
  try:
    (opts, args) = getopt.getopt(sys.argv[1:], "h:r:n:")
  except getopt.GetoptError:
    usage()
  (archiveurl, puburl, n) = (None, None, 20)
  for (k, v) in opts:
    if k == "-h": archiveurl = v
    elif k == "-r": puburl = v
    elif k == "-n": n = int(v)
  if len(args) != 2:
    usage()
  (repname, editurl) = args
  repository = Repository(repname)
  items = repository.recent_items()
  items.sort(lambda (ida,a0,a1),(idb,b0,b1): cmp(b0.modtime, a0.modtime))
  items = items[:n]
  info = repository.get_info()
  if puburl:
    export_rss(info, items, editurl, puburl)
  elif archiveurl:
    export_html(info, items, editurl, archiveurl)

if __name__ == '__main__': main()
