#!/usr/bin/env python
import sys, os, re, urllib, time
from xml.sax import handler, parse
from repository import Repository, TransactionError
from utils import remsp, ripalltags


##  RSSReader
##
class RSSReader(handler.ContentHandler):
  
  def __init__(self, repository, htmltagrip=False):
    self.repository = repository
    self.phase = 0
    self.s = []
    self.dic = {}
    self.htmltagrip = htmltagrip
    return handler.ContentHandler.__init__(self)
  
  def startElement(self, name, attrs):
    if name == 'item':
      assert self.phase == 0
      self.phase = 1
    elif self.phase == 1 and name in ('title', 'link', 'description'):
      self.phase = 2
      self.s = []
    return
  
  def endElement(self, name):
    if name == 'item':
      assert self.phase == 1
      (title, link, description) = [ self.dic.get(k, '') for k in ('title', 'link', 'description') ]
      if title and link:
        try:
          if self.htmltagrip:
            description = ripalltags(description)
          self.repository.new_item(int(time.time()), title, link, description)
          print >>sys.stderr, 'added:', link
        except TransactionError, e:
          print >>sys.stderr, 'ignored:', e, link
      self.phase = 0
    elif self.phase == 2:
      self.dic[name] = remsp(''.join(self.s))
      self.phase = 1
    return
  
  def characters(self, ch):
    if self.phase == 2:
      self.s.append(ch)
    return


# main
def main():
  import getopt
  def usage():
    print "usage: inject.py [-h] repdir url"
    sys.exit(2)
  try:
    (opts, args) = getopt.getopt(sys.argv[1:], "dh")
  except getopt.GetoptError:
    usage()
  (debug, htmltagrip) = (False, False)
  for (k,v) in opts:
    if k == '-d': debug = True
    elif k == '-h': htmltagrip = True
  if len(args) != 2:
    usage()
  (dirname, url) = args
  repository = Repository(dirname)
  reader = RSSReader(repository, htmltagrip)
  fp = urllib.urlopen(url)
  parse(fp, reader)
  fp.close()
  return

if __name__ == "__main__": main()
