#!/usr/bin/env python
# -*- encoding: euc-jp -*-

import cgitb; cgitb.enable()
import sys, os, re, cgi, time
from repository import Repository, TransactionError
from utils import *
stderr = sys.stderr

REPOSITORIES = [ 'lwn', 'slashdot' ]
HELP_URL = 'http://traiss.tabesugi.net/help.html'
LIST_URL = 'http://traiss.tabesugi.net/%s.html'
ARCHIVE_URL = 'http://traiss.tabesugi.net/archives/%s/'

STYLES = '''<style><!--
h2.headline { letter-spacing:0.2em; text-align:left; padding-bottom:4pt; border-bottom:4pt solid darkblue; width:50%; }
a:hover { color:red; }
.s0 { background:yellow; color:black; text-decoration:underline; }
.s1 { color:red; }
.s2 { color:#c08; }
.s3 { color:#080; }
.s4 { color:black; }
.nav { font-size: 80%; }
.d { color:#f60; text-decoration:underline; }
.g { background:#ddf; }
.b { font-weight:bold; color:red; }
.c { background:#8cf; font-weight:bold; color:black; }
.e { background:red; font-weight:bold; color:white; }
--></style>'''


##  Preferences
##
class Preferences:
  
  DEFAULTS = {
    'edit': '1',
    'sort': '0',
    'editor': '',
    }
  
  def __init__(self, form):
    self.value = {}
    for k in form.keys():
      if k in self.DEFAULTS:
        self.value[k] = form.getvalue(k, '')
    return

  def getvalue(self, k):
    try:
      return self.value[k]
    except KeyError:
      return self.DEFAULTS[k]

  def makeurl(self, url='', **changes):
    dic = self.value.copy()
    dic.update(changes)
    seq = [ '%s=%s' % (k,urlquote(v)) for (k,v) in dic.iteritems()
            if (k not in self.DEFAULTS) or (v != self.DEFAULTS[k]) ]
    if seq:
      return url+'?'+'&'.join(seq)
    else:
      return url or '?'

  def makeparams(self, elim=[], **changes):
    dic = self.value.copy()
    dic.update(changes)
    return ''.join([ '<input type=hidden name="%s" value="%s">' % (k,urlquote(v))
                     for (k,v) in dic.iteritems()
                     if (k not in elim) and (k not in self.DEFAULTS or v != self.DEFAULTS[k]) ])


##  Traiss
##
class Traiss:

  PATHPAT = re.compile(r'/([a-zA-Z0-9]+/)([0-9a-f]+)?')
  def __init__(self):
    self.path_info = os.environ.get('PATH_INFO', '')
    self.request_method = os.environ.get('REQUEST_METHOD', 'GET')
    self.form = cgi.FieldStorage()
    self.pref = Preferences(self.form)
    self.curtime = int(time.time())
    self.pagetitle = ''
    return

  def run(self):
    m = self.PATHPAT.match(self.path_info)
    if (self.path_info != '/') and (not m):
      print 'HTTP/1.0 404 Not Found'
      print 'Content-type: text/html'
      print
      print '<html><body>file does not exist</body></html>'
      print >>stderr, 'traiss: not found: %r' % self.path_info
      return
    print 'HTTP/1.0 200 OK'
    print 'Content-type: text/html; charset=%s' % HTML_ENCODING
    #print 'Pragma: no-cache\nCache-Control: no-cache\nExpires: -1'
    print 
    (repname,itemid) = (None, None)
    if m:
      (repname,itemid) = (m.group(1)[:-1], m.group(2))
    if repname not in REPOSITORIES:
      self.header()
      self.list_repositories()
      print u'<p> ץȤ򤷤Ƥ <hr noshade>'
      self.footer()
    else:
      # repository specified
      repository = Repository(repname)
      if self.pref.getvalue('edit') == '1':
        self.select_item(repository, itemid)
      else:
        self.display_item(repository, itemid)
    return

  def select_item(self, repository, itemid):
    print >>stderr, 'traiss: select:', itemid
    msg = u'򤷤Ƥ'
    if itemid:
      # item specified
      try:
        item = repository.get_item(itemid)
        revision = safeint(self.form.getvalue('rev'), -1)
        editor = remsp(decode_form(self.form, 'editor', limit=100))
        # item obtained successfully.
        (msg, ent0, ent1, entold, history, title, description, published, highlight) = \
              self.process_query(item, editor, revision)
        self.pagetitle = '/%s/%s' % (repository.name, title)
        self.header()
        self.list_repositories(repository.name)
        print '<p><form method=post action="/%s/%s"><input type=hidden name=cmd value=save>' % \
              (repository.name, itemid)
        print '<table border cellspacing="0" cellpadding="8"><tr valign=top>'
        self.list_items(repository)
        self.edit_item1(msg, ent0, ent1, entold, history, highlight,
                        title, description, published, editor)
        print '</tr></table></form>'
        print u'<p><small><strong>:</strong> Խ줿Ƥϸޤλ'
        print u'<a href="%s">[ܤϤ]</a></small>' % HELP_URL
        self.footer()
        return
      except TransactionError, e:
        msg = u'<span class=e>å˼ԤޤĤʤïԽƤǽޤ⤦٥ɤƤߤƤ</span>'
        print >>stderr, 'traiss: FATAL: lock failed', itemid
    # item not found or not specified.
    self.pagetitle = '/%s/' % repository.name
    self.header()
    self.list_repositories(repository.name)
    print '<p><table border cellspacing="0" cellpadding="8"><tr valign=top>'
    self.list_items(repository)
    print '<td>%s</td></tr></table>' % msg
    print u'<p><small><strong>:</strong> Խ줿Ƥϸޤλ'
    print u'<a href="%s">[ܤϤ]</a></small>' % HELP_URL
    self.footer()
    return

  def process_query(self, item, editor, revision):
    (ent0, ent1, entold, history) = item.getent(revision)
    msg = ''
    highlight = 0
    title0 = dump_raw(ent1.title)
    description0 = dump_raw(ent1.description)
    published0 = (ent1.flag == 'P')
    if self.form.getvalue('cmd') == 'save' and self.request_method == 'POST':
      # process form
      title1 = riptags(decode_form(self.form, 'title', limit=1000))
      description1 = riptags(decode_form(self.form, 'description', limit=10000))
      published1 = (self.form.getvalue('publish') == 'on')
      if title0 and not title1:
        msg = u'<span class=e>顼: ȥ뤬ˤʤäƤޤ</span>'
      elif description0 and not description1:
        msg = u'<span class=e>顼: ܸˤʤäƤޤ</span>'
      elif not editor:
        msg = u'<span class=e>顼: Խ̾ϤƤ</span>'
      elif title0 != title1 or description0 != description1 or published0 != published1:
        flag = 'N'
        if published1:
          flag = 'P'
        try:
          origent = (ent1.revision, ent1.modtime, ent1.editor)
          ent1 = ent1.item.commit(title1, description1, editor, flag, self.curtime)
          history.insert(0, origent)
          highlight = self.curtime
          msg = u'<span class=c>ѹȿǤޤ</span>'
        except TransactionError, e:
          msg = u'<span class=e>顼: å˼ԤޤïεԽƤǽޤ⤦٥ɤƤߤƤ</span>'
          print >>stderr, 'traiss: FATAL: lock failed', item.itemid
      print >>stderr, 'traiss: saved:', item.itemid, repr(editor)
      title0 = (title1 or title0)
      description0 = (description1 or description0)
      published0 = published1
    return (msg, ent0, ent1, entold, history,
            title0, description0, published0, highlight)

  def header(self):
    print '<html><head><title>Traiss%s</title>' % htmlquote(self.pagetitle)
    print '<meta name="robots" content="nofollow">'
    print STYLES
    print '</head><body>'
    print '<h2 class="headline">Traiss</h2>'
    print '<div align=right class=nav><a href="http://traiss.tabesugi.net/">http://traiss.tabesugi.net/</a></div>'
    return

  def list_repositories(self, repname=None):
    r = []
    for name in REPOSITORIES:
      if repname == name:
        r.append('<strong>%s</strong>' % name)
      else:
        r.append('<a href="%s">%s</a>' % (self.pref.makeurl('/%s/' % name), name))
    print u'<p><small>ץ: %s</small>' % ', '.join(r)
    return

  def list_items(self, repository, n=20):
    print >>stderr, 'traiss: list:', repository.name
    items0 = repository.recent_items()
    if self.pref.getvalue('sort') == '0':
      (label,altlabel,altorder) = (u'', u'ܸι˥', '1')
      items = [ (ent0.modtime, itemid, ent1) for (itemid,ent0,ent1) in items0 ]
    else:
      (label,altlabel,altorder) = (u'', u'ʸμ˥', '0')
      items = [ (ent1.modtime, itemid, ent1) for (itemid,ent0,ent1) in items0 ]
    items.sort()
    items.reverse()
    items = items[:n]
    (rep_title, rep_link, rep_description) = repository.get_info()
    print '<td rowspan="3" width="25%">'
    print '<p><small><strong><a href="%s">%s</a></strong> - %s</small><p><table>' % \
          (LIST_URL % repository.name, htmlquote(rep_title), htmlquote(rep_description))
    print '<tr><td><small>%s</small></td><td align=right><small>' % label
    print '<a href="%s">[%s]</a></small></td></tr>' % (self.pref.makeurl(sort=altorder), altlabel)
    for (i,(modtime,itemid,ent1)) in enumerate(items):
      s = ''
      if ent1.flag == 'P':
        s = ' <span class="b">*</span>'
      print '<tr valign=top><td><small>%s</small></td><td><small>%s' % (repr_time(modtime, self.curtime), s)
      print '<a href="%s">%s</a>' % (self.pref.makeurl(itemid), htmlquote(dump_raw(ent1.title)))
      print '<em>%s</em></small></tr>' % htmlquote(ent1.editor)
    print u'<tr><td></td><td><small><div align=right><a href="%s">[ε...]</a></div><span class="b">*</span>: RSS</small></td></tr></table></td>' % (ARCHIVE_URL % repository.name)
    return items

  def edit_item1(self, msg, ent0, ent1, entold, history, highlight,
                 title, description, published, editor):
    print '<td height="1">'
    title_src = ent1.title
    if entold:
      (title_src, title_old) = diff(ent1.title, entold.title)
      print '<span class=g><strong>Rev. %d: %s</strong></span><br>' % (entold.revision, dump_style(title_old, 0))
    print '<font size="+2"><strong>%s</strong></font>' % dump_style(title_src, self.curtime, highlight)
    print '<p><a href="%s"><strong>%s</strong></a>' % (htmlquote(ent0.link), htmlquote(dump_raw(ent0.title)))
    print u'<br><small>: %s, : %s <em>%s</em>' % \
          (repr_time(ent0.modtime, self.curtime), 
           repr_time(ent1.modtime, self.curtime), htmlquote(ent1.editor))
    if ent1.flag == 'P':
      print u'&nbsp; <span class=b>RSS</span>'
    print u'&nbsp; <a href="%s">[ץӥ塼]</a>' % self.pref.makeurl(edit='0')
    # editable title
    print u'</small><p><small>ȥ (<u>T</u>):</small><br>'
    print '<textarea accesskey=t name=title style="width: 100%;" cols="50" wrap=soft rows="1">'
    print htmlquote(title)
    print '</textarea></td></tr><tr valign=top>'
    # source description
    print u'<td height="1"><p><small><strong>ʸ:</strong></small> <tt>%s</tt>' % htmlquote(dump_raw(ent0.description))
    print '</td></tr><tr valign=top><td>'
    description_src = ent1.description
    if entold:
      (description_src, description_old) = diff(ent1.description, entold.description)
      print '<blockquote class=g><small><strong>Rev. %d:</strong>' % entold.revision
      print '<tt>%s</tt></small></blockquote>' % dump_style(description_old, 0)
    print u'<p><small><strong>ܸ:</strong></small> '
    print '<tt>%s</tt>' % dump_style(description_src, self.curtime, highlight)
    if msg:
      print '<p>%s' % msg
    height = min(len(description)/60+1, 20)
    # editable description
    print u'<p><small> (<u>D</u>):</small><br>'
    #print u'(Ѳǽ: a, b, u, i, s, q, p, strong, em, code, kbd, span, br)</small><br>'
    print '<textarea accesskey=d name=description style="width: 100%%;" cols="80" wrap=soft rows="%d">' % height
    print htmlquote(description)
    print '</textarea><div align=right>'
    s = ''
    if published:
      s = ' checked'
    print u'<label for=publish>(<u>P</u>): <input accesskey=p type=checkbox name=publish id=publish%s></label>' % s
    print u'&nbsp; Խ(<u>E</u>): <input accesskey=e size="10" type=text name=editor value="%s">' % htmlquote(editor)
    print u'&nbsp;<input accesskey=s type=submit value="¸ (S)">'+self.pref.makeparams(['editor'])
    print u'</div><small>ѹ줿ʬ: '
    print u', '.join([ u'<span class=%s></span> %dְ' % (c,t) 
                       for (c,t) in (('s1',1), ('s2',6), ('s3',24)) ]), '</small>'
    if entold:
      print u', <span class=d></span> Rev. %d Ȥκʬ' % entold.revision
    if history:
      print u'<p><small>ΥС:<ul>'
      for (rev,modtime,editor) in history:
        if entold and rev == entold.revision:
          print '<li> <strong>Rev.%d : %s <em>%s</em></strong>' % \
                (rev, repr_time(modtime, self.curtime), htmlquote(editor))
        else:
          print '<li> <a href="%s">Rev.%d</a> : %s <em>%s</em>' % \
                (self.pref.makeurl(rev=str(rev)), rev, repr_time(modtime, self.curtime), htmlquote(editor))
      print '</ul></small>'
    print '</td>'
    return

  def footer(self):
    print '<address>Traiss-%s</address></body></html>' % TRAISS_VERSION

  def display_item(self, repository, itemid):
    print >>stderr, 'traiss: disp:', repository.name, itemid
    msg = u'򤵤Ƥޤ'
    if itemid:
      # item specified
      try:
        item = repository.get_item(itemid)
        (ent0, ent1, ent2, history) = item.getent(nhist=0)
        (rep_title, rep_link, rep_description) = repository.get_info()
        # item obtained successfully.
        title = dump_raw(ent1.title)
        pagetitle = u'%s: %s' % (repository.name, title)
        print '<html><head><title>%s</title>' % htmlquote(pagetitle)
        print '<meta name="robots" content="nofollow">'
        print '''<style><!--
h2 { text-align:left; padding-left:8pt; padding-bottom:4pt; border-left:6pt solid brown; border-bottom:2pt solid brown; }
body { line-height: 130%; }
--></style>'''
        print '</head><body><p><small><a href="%s">%s</a></small>' % (LIST_URL % repository.name, htmlquote(rep_title))
        print '<h2>%s</h2>' % htmlquote(title)
        print u'<p><strong><a href="%s">%s</a></strong>&nbsp;' % (htmlquote(ent0.link), dump_raw(ent0.title))
        print u'<small>: %s</small>' % repr_time(ent0.modtime, self.curtime)
        if ent1.description:
          print '<p>%s' % dump_raw(ent1.description)
        print u'<div align=right><a href="%s">[Խ̤]</a></div>' % self.pref.makeurl(edit='1')
        print '<hr><address>Generated by <a href="http://traiss.tabesugi.net/">Traiss-%s</a></address>' % TRAISS_VERSION
        print '</body></html>'
        return
      except TransactionError, e:
        pagetitle = 'Error'
        msg = u'å˼Ԥޤ⤦٥ɤƤߤƤ'
        print >>stderr, 'traiss: FATAL: lock failed', itemid
    print '<html><body>%s</body></html>' % htmlquote(msg)
    return
    

#
def main(): Traiss().run()
